//	warpresize - Edge-directed resampling filter for VirtualDub
//	Copyright (C) 2005 Avery Lee
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <vector>
#include "vdfilter.h"

///////////////////////////////////////////////////////////////////////////

VDFilterBase::VDFilterBase(FilterActivation *fa_, const FilterFunctions *ff_)
	: fa(fa_)
	, ff(ff_)
{
}

VDFilterBase::~VDFilterBase() {
}

///////////////////////////////////////////////////////////////////////////

bool VDFilterBase::Init() {
	return true;
}

void VDFilterBase::Start() {
}

void VDFilterBase::End() {
}

bool VDFilterBase::Configure(HWND hwnd) {
	return false;
}

void VDFilterBase::GetSettingString(char *buf, int maxlen) {
}

void VDFilterBase::GetScriptString(char *buf, int maxlen) {
}

int VDFilterBase::Serialize(char *buf, int maxbuf) {
	return 0;
}

int VDFilterBase::Deserialize(const char *buf, int maxbuf) {
	return 0;
}

///////////////////////////////////////////////////////////////////////////

INT_PTR CALLBACK VDFilterBase::StaticDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
	VDFilterBase *pThis;

	if (msg == WM_INITDIALOG) {
		SetWindowLongPtr(hdlg, DWL_USER, (LONG_PTR)lParam);
		pThis = (VDFilterBase *)lParam;
		pThis->mhdlg = hdlg;
	} else {
		pThis = (VDFilterBase *)GetWindowLongPtr(hdlg, DWL_USER);
	}

	if (pThis)
		return pThis->DlgProc(msg, wParam, lParam);
	else
		return FALSE;
}

INT_PTR VDFilterBase::DlgProc(UINT msg, WPARAM wParam, LPARAM lParam) {
	return FALSE;
}

bool VDFilterBase::DoConfigureDialog(HWND hwnd, LPCTSTR tmplName) {
	MEMORY_BASIC_INFORMATION memInfo;
	VirtualQuery("", &memInfo, sizeof memInfo);
	return 0 != DialogBoxParam((HMODULE)memInfo.AllocationBase, tmplName, hwnd, StaticDlgProc, (LONG_PTR)this);
}

///////////////////////////////////////////////////////////////////////////

void __cdecl VDFilterBase::FilterDeinit   (FilterActivation *fa, const FilterFunctions *ff) {
	reinterpret_cast<VDFilterBase *>(fa->filter_data)->~VDFilterBase();
}

int  __cdecl VDFilterBase::FilterRun      (const FilterActivation *fa, const FilterFunctions *ff) {
	VDFilterBase *pThis = reinterpret_cast<VDFilterBase *>(fa->filter_data);

	pThis->fa		= const_cast<FilterActivation *>(fa);

	return !pThis->Run();
}

long __cdecl VDFilterBase::FilterParam    (FilterActivation *fa, const FilterFunctions *ff) {
	VDFilterBase *pThis = reinterpret_cast<VDFilterBase *>(fa->filter_data);

	pThis->fa		= fa;

	return pThis->GetParams();
}

int  __cdecl VDFilterBase::FilterConfig   (FilterActivation *fa, const FilterFunctions *ff, HWND hwnd) {
	VDFilterBase *pThis = reinterpret_cast<VDFilterBase *>(fa->filter_data);

	pThis->fa		= fa;

	return pThis->Configure(hwnd);
}

int  __cdecl VDFilterBase::FilterStart    (FilterActivation *fa, const FilterFunctions *ff) {
	VDFilterBase *pThis = reinterpret_cast<VDFilterBase *>(fa->filter_data);

	pThis->fa		= fa;

	pThis->Start();
	return 0;
}

int  __cdecl VDFilterBase::FilterEnd      (FilterActivation *fa, const FilterFunctions *ff) {
	VDFilterBase *pThis = reinterpret_cast<VDFilterBase *>(fa->filter_data);

	pThis->fa		= fa;

	pThis->End();
	return 0;
}

bool __cdecl VDFilterBase::FilterScriptStr(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen) {
	VDFilterBase *pThis = reinterpret_cast<VDFilterBase *>(fa->filter_data);

	pThis->fa		= fa;

	pThis->GetScriptString(buf, buflen);

	return true;
}

void __cdecl VDFilterBase::FilterString2  (const FilterActivation *fa, const FilterFunctions *ff, char *buf, int maxlen) {
	VDFilterBase *pThis = reinterpret_cast<VDFilterBase *>(fa->filter_data);

	pThis->fa		= const_cast<FilterActivation *>(fa);

	pThis->GetSettingString(buf, maxlen);
}

int  __cdecl VDFilterBase::FilterSerialize    (FilterActivation *fa, const FilterFunctions *ff, char *buf, int maxbuf) {
	VDFilterBase *pThis = reinterpret_cast<VDFilterBase *>(fa->filter_data);

	pThis->fa		= fa;

	return pThis->Serialize(buf, maxbuf);
}

void __cdecl VDFilterBase::FilterDeserialize  (FilterActivation *fa, const FilterFunctions *ff, const char *buf, int maxbuf) {
	VDFilterBase *pThis = reinterpret_cast<VDFilterBase *>(fa->filter_data);

	pThis->fa		= fa;

	pThis->Deserialize(buf, maxbuf);
}
